@@ -1,6 +1,6 @@
 /* ============================================================
 * QuiteRSS is a open-source cross-platform RSS/Atom news feeds reader
-* Copyright (C) 2011-2013 QuiteRSS Team <quiterssteam@gmail.com>
+* Copyright (C) 2011-2014 QuiteRSS Team <quiterssteam@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -111,6 +111,9 @@ class QyurSqlTreeModelPrivate {
   mutable QMap<int,UserData*> id2UserData;
   mutable QMap<QModelIndex,QModelIndex> proxy2source;
   mutable QMap<int,int> proxyColumn2Original;
+
+  QMap<QObject*,QSet<int> > expandedSet;
+
 public:
   QSqlTableModelEx sourceModel;
   int indexOfId, indexOfParid, rootParentId;
@@ -198,6 +201,10 @@ QyurSqlTreeModel::QyurSqlTreeModel(const QString& tableName,
   }
   d->proxyColumn2Original[0] = d->sourceModel.record().indexOf(fieldNames[0]);
   d->proxyColumn2Original[d->sourceModel.record().indexOf(fieldNames[0])] = 0;
+
+  d->sourceModel.setSort(d->indexOfParid, (Qt::SortOrder)DROP_MULTIORDER);
+  d->sourceModel.setSort(d->sourceModel.fieldIndex("rowToParent"), Qt::AscendingOrder);
+  refresh();
 }
 
 int QyurSqlTreeModel::originalColumnByProxy(int proxyColumn) const {
@@ -487,7 +494,6 @@ int QyurSqlTreeModel::getRootParentId() const {
 class QyurSqlTreeViewPrivate {
 public:
   QyurSqlTreeViewPrivate();
-  QList<QyurIntPair> expandedNodeList;
 };
 
 QyurSqlTreeViewPrivate::QyurSqlTreeViewPrivate()
@@ -495,67 +501,38 @@ QyurSqlTreeViewPrivate::QyurSqlTreeViewPrivate()
 }
 
 QyurSqlTreeView::QyurSqlTreeView(QWidget * parent)
-  : QTreeView(parent),d_ptr(new QyurSqlTreeViewPrivate)
+  : QTreeView(parent)
+  , d_ptr(new QyurSqlTreeViewPrivate)
+  , sourceModel_(0)
 {
   setSortingEnabled(false);
-  disconnect(header(), SIGNAL(sectionClicked(int)), this, SLOT(sortByColumn(int)));
-  connect(header(), SIGNAL(sectionClicked(int)), SLOT(slotSortByColumnAndSelect(int)));
   setContextMenuPolicy(Qt::CustomContextMenu);
   setSelectionMode(QAbstractItemView::ExtendedSelection);
-//  QObject::connect(this,SIGNAL(expanded(const QModelIndex&)),SLOT(slotAddExpanded(const QModelIndex&)));
-//  QObject::connect(this,SIGNAL(collapsed(const QModelIndex&)),SLOT(slotRemoveExpanded(const QModelIndex&)));
 }
 
-QyurSqlTreeView::~QyurSqlTreeView() {
+QyurSqlTreeView::~QyurSqlTreeView()
+{
   delete d_ptr;
 }
 
-void QyurSqlTreeView::slotAddExpanded(const QModelIndex& index) {
-  Q_D(QyurSqlTreeView);
-  d->expandedNodeList.append(QyurIntPair( ((QyurSqlTreeModel*) model())->getIdByIndex(index), ((QyurSqlTreeModel*) model())->getParidByIndex(index)) );
+void QyurSqlTreeView::setSourceModel(QyurSqlTreeModel *model)
+{
+  sourceModel_ = model;
 }
 
-void QyurSqlTreeView::slotRemoveExpanded(const QModelIndex& index) {
-  Q_D(QyurSqlTreeView);
-  d->expandedNodeList.removeAll(QyurIntPair( ((QyurSqlTreeModel*) model())->getIdByIndex(index), ((QyurSqlTreeModel*) model())->getParidByIndex(index)) );
-}
-
-void QyurSqlTreeView::slotSortByColumnAndSelect(int column) {
-  int id= -1;
-  if (selectionModel()->selectedRows().size()>0) {
-    id = selectionModel()->selectedRows()[0].sibling(selectionModel()->selectedRows()[0].row(),((QyurSqlTreeModel*) model())->getFieldPosition(QyurSqlTreeModel::Id)).data().toInt();
-  }
-  QTreeView::sortByColumn(column);
-  restore(id);
-}
-
-void QyurSqlTreeView::restore(int id) {
-  Q_D(QyurSqlTreeView);
-  if (id!=-1) {
-    QModelIndex selectedIndex= ((QyurSqlTreeModel*) model())->getIndexById(id);
-    scrollTo(selectedIndex);
-    selectionModel()->select(selectedIndex,QItemSelectionModel::ClearAndSelect|QItemSelectionModel::Rows);
-  }
-  foreach(QyurIntPair pair, d->expandedNodeList)
-    setExpanded(((QyurSqlTreeModel*) model())->getIndexById(pair.first),true);
-}
-
-void QyurSqlTreeView::restoreExpanded() {
-  Q_D(QyurSqlTreeView);
-  foreach(QyurIntPair pair, d->expandedNodeList)
-    setExpanded(((QyurSqlTreeModel*) model())->getIndexById(pair.first),true);
-}
-
-void QyurSqlTreeView::onInsertRow(QSqlRecord&) {
+void QyurSqlTreeView::onInsertRow(QSqlRecord&)
+{
 
 }
 
-void QyurSqlTreeView::setColumnHidden(const QString& column, bool hide) {
-  //QTreeView::setColumnHidden(((QyurSqlTreeModel*) model())->proxyColumnByOriginal(column),hide);
+void QyurSqlTreeView::setColumnHidden(const QString& column, bool hide)
+{
   QTreeView::setColumnHidden(columnIndex(column),hide);
 }
 
-int QyurSqlTreeView::columnIndex(const QString& fieldName) const {
-  return ((QyurSqlTreeModel*) model())->proxyColumnByOriginal(fieldName);
+int QyurSqlTreeView::columnIndex(const QString& fieldName) const
+{
+  return sourceModel_->proxyColumnByOriginal(fieldName);
 }
+
 #include "qyursqltreeview.moc"
