@@ -1,13 +1,9 @@
 #include "VersionRev.h"
 
-#if defined(Q_OS_WIN)
-#define PORTABLE       1
-#endif
+#define STRDATE           "03.09.2014\0"
+#define STRPRODUCTVER     "0.17.0\0"
 
-#define STRDATE           "30.08.2013\0"
-#define STRPRODUCTVER     "0.13.3\0"
-
-#define VERSION           0,13,3
+#define VERSION           0,17,0
 #define PRODUCTVER        VERSION,0
 #define FILEVER           VERSION,VCS_REVISION
 
