@@ -1,6 +1,6 @@
 /* ============================================================
 * QuiteRSS is a open-source cross-platform RSS/Atom news feeds reader
-* Copyright (C) 2011-2013 QuiteRSS Team <quiterssteam@gmail.com>
+* Copyright (C) 2011-2014 QuiteRSS Team <quiterssteam@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,12 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
 #include "addfeedwizard.h"
+
+#include "mainapplication.h"
 #include "addfolderdialog.h"
 #include "authenticationdialog.h"
+#include "toolbutton.h"
+#include "settings.h"
 
 #include <QDomDocument>
 
@@ -36,8 +40,7 @@ AddFeedWizard::AddFeedWizard(QWidget *parent, int curFolderId)
   addPage(createUrlFeedPage());
   addPage(createNameFeedPage());
 
-  persistentUpdateThread_ = new UpdateThread(this);
-  persistentParseThread_ = new ParseThread(this);
+  updateFeeds_ = new UpdateFeeds(this, true);
 
   connect(button(QWizard::BackButton), SIGNAL(clicked()),
           this, SLOT(backButtonClicked()));
@@ -49,14 +52,17 @@ AddFeedWizard::AddFeedWizard(QWidget *parent, int curFolderId)
           SLOT(slotCurrentIdChanged(int)),
           Qt::QueuedConnection);
   resize(400, 300);
+
+  Settings settings;
+  restoreGeometry(settings.value("addFeedWizard/geometry").toByteArray());
 }
 
 AddFeedWizard::~AddFeedWizard()
 {
-  persistentUpdateThread_->quit();
-  persistentParseThread_->quit();
-  while (persistentUpdateThread_->isRunning());
-  while (persistentParseThread_->isRunning());
+  Settings settings;
+  settings.setValue("addFeedWizard/geometry", saveGeometry());
+
+  delete updateFeeds_;
 }
 
 /*virtual*/ void AddFeedWizard::done(int result)
@@ -77,7 +83,8 @@ void AddFeedWizard::changeEvent(QEvent *event)
     if (clipboardStr.contains("http://", Qt::CaseInsensitive) ||
         clipboardStr.contains("https://", Qt::CaseInsensitive) ||
         clipboardStr.contains("www.", Qt::CaseInsensitive) ||
-        clipboardStr.contains("feed://", Qt::CaseInsensitive)) {
+        clipboardStr.contains("feed://", Qt::CaseInsensitive) ||
+        clipboardStr.contains("file://", Qt::CaseInsensitive)) {
       urlFeedEdit_->setText(clipboard_->text());
       urlFeedEdit_->selectAll();
       urlFeedEdit_->setFocus();
@@ -216,7 +223,7 @@ QWizardPage *AddFeedWizard::createNameFeedPage()
   foldersTree_->expandAll();
   foldersTree_->sortByColumn(0, Qt::AscendingOrder);
 
-  QToolButton *newFolderButton = new QToolButton(this);
+  ToolButton *newFolderButton = new ToolButton(this);
   newFolderButton->setIcon(QIcon(":/images/addT"));
   newFolderButton->setToolTip(tr("New Folder..."));
   newFolderButton->setAutoRaise(true);
@@ -247,6 +254,11 @@ QWizardPage *AddFeedWizard::createNameFeedPage()
   return page;
 }
 
+void AddFeedWizard::setUrlFeed(const QString &feedUrl)
+{
+  urlFeedEdit_->setText(feedUrl);
+}
+
 void AddFeedWizard::urlFeedEditChanged(const QString& text)
 {
   button(QWizard::NextButton)->setEnabled(
@@ -303,14 +315,28 @@ void AddFeedWizard::finishButtonClicked()
 void AddFeedWizard::addFeed()
 {
   // Set URL-schema for URL-address "http://" or leave it "https://"
+  feedUrlString_ = urlFeedEdit_->text().simplified();
+  if (feedUrlString_.contains("feed:", Qt::CaseInsensitive)) {
+    if (feedUrlString_.contains("https://", Qt::CaseInsensitive)) {
+      feedUrlString_.remove(0, 5);
+      urlFeedEdit_->setText(feedUrlString_);
+    } else {
+      feedUrlString_.remove(0, 7);
+      urlFeedEdit_->setText("http://" + feedUrlString_);
+    }
+  }
   QUrl feedUrl(urlFeedEdit_->text().simplified());
-  if (feedUrl.scheme().toLower() != "https") {
-    feedUrl.setScheme("http");
+  if (feedUrl.scheme().isEmpty()) {
+    feedUrl.setUrl("http://" % urlFeedEdit_->text().simplified());
   }
   feedUrlString_ = feedUrl.toString();
   urlFeedEdit_->setText(feedUrlString_);
 
-  if (feedUrl.host().isEmpty()) {
+#if QT_VERSION >= 0x040800
+  if (feedUrl.host().isEmpty() && !feedUrl.isLocalFile()) {
+#else
+  if (feedUrl.host().isEmpty() && (feedUrl.scheme() != "file")) {
+#endif
     textWarning->setText(tr("URL error!"));
     warningWidget_->setVisible(true);
     return;
@@ -359,6 +385,26 @@ void AddFeedWizard::addFeed()
     feedId_ = q.lastInsertId().toInt();
     q.finish();
 
+    if (feedUrlString_.contains(":COOKIE:", Qt::CaseInsensitive)) {
+      int index = feedUrlString_.lastIndexOf(":COOKIE:", -1, Qt::CaseInsensitive);
+      QString cookieStr = feedUrlString_.right(feedUrlString_.length() - index - 8);
+      QStringList cookieStrList = cookieStr.split(";");
+
+      QList<QNetworkCookie> loadedCookies;
+      foreach (QString cookieStr, cookieStrList) {
+        const QList<QNetworkCookie> &cookieList = QNetworkCookie::parseCookies(cookieStr.toUtf8());
+        if (cookieList.isEmpty()) {
+          continue;
+        }
+        QNetworkCookie cookie = cookieList.at(0);
+        QDateTime date = QDateTime::currentDateTime();
+        date = date.addYears(35);
+        cookie.setExpirationDate(date);
+        loadedCookies.append(cookie);
+      }
+      mainApp->cookieJar()->setCookiesFromUrl(loadedCookies, feedUrlString_);
+    }
+
     emit signalRequestUrl(feedId_, feedUrlString_, QDateTime(), userInfo);
   }
 }
@@ -411,10 +457,9 @@ void AddFeedWizard::slotProgressBarUpdate()
     QTimer::singleShot(250, this, SLOT(slotProgressBarUpdate()));
 }
 
-void AddFeedWizard::getUrlDone(const int &result, const int &feedId,
-                               const QString &feedUrlStr, const QString &error,
-                               const QByteArray &data, const QDateTime &dtReply,
-                               const QString &codecName)
+void AddFeedWizard::getUrlDone(int result, int feedId, QString feedUrlStr,
+                               QString error, QByteArray data,
+                               QDateTime dtReply, QString codecName)
 {
   if (!data.isEmpty()) {
     bool isFeed = false;
@@ -423,8 +468,9 @@ void AddFeedWizard::getUrlDone(const int &result, const int &feedId,
     int errorColumn;
     QDomDocument doc("parseDoc");
     if (!doc.setContent(data, false, &errorStr, &errorLine, &errorColumn)) {
-      qDebug() << QString("Parse data error: line %1, column %2: %3").
-                  arg(errorLine).arg(errorColumn).arg(errorStr);
+      qWarning() << QString("Parse data error (1): url %1, id %2, line %3, column %4: %5").
+                    arg(feedUrlStr).arg(feedId).
+                    arg(errorLine).arg(errorColumn).arg(errorStr);
     } else {
       QDomElement docElem = doc.documentElement();
       if ((docElem.tagName() == "rss") || (docElem.tagName() == "feed") ||
@@ -444,6 +490,7 @@ void AddFeedWizard::getUrlDone(const int &result, const int &feedId,
         pos = rx.indexIn(str);
         if (pos > -1) {
           QString linkFeedString = rx.cap(1);
+          linkFeedString.replace("&amp;", "&", Qt::CaseInsensitive);
           QUrl url(linkFeedString);
           QUrl feedUrl(feedUrlStr);
           if (url.host().isEmpty()) {
@@ -512,6 +559,8 @@ void AddFeedWizard::getUrlDone(const int &result, const int &feedId,
     else
       textWarning->setText(tr("Request failed!"));
     warningWidget_->setVisible(true);
+    qWarning() << QString("Request failed: result = %1, error - %2, url - %3").
+                  arg(result).arg(error).arg(feedUrlStr);
 
     deleteFeed();
     progressBar_->hide();
@@ -521,8 +570,7 @@ void AddFeedWizard::getUrlDone(const int &result, const int &feedId,
   }
 }
 
-void AddFeedWizard::slotUpdateFeed(const int &feedId, const bool &,
-                                   const int &newCount, const QString &)
+void AddFeedWizard::slotUpdateFeed(int feedId, bool, int newCount, QString)
 {
   qDebug() << "ParseDone: " << feedUrlString_;
   selectedPage = true;
@@ -657,22 +705,3 @@ void AddFeedWizard::newFolder()
 
   delete addFolderDialog;
 }
-
-void AddFeedWizard::slotAuthentication(QNetworkReply *reply, QAuthenticator *auth)
-{
-  AuthenticationDialog *authenticationDialog =
-      new AuthenticationDialog(this, reply->url(), auth);
-
-  if (!authenticationDialog->save_->isChecked())
-    authenticationDialog->exec();
-
-  delete authenticationDialog;
-}
-
-void AddFeedWizard::slotFeedCountsUpdate(FeedCountStruct)
-{
-}
-
-void AddFeedWizard::setStatusFeed(const int &, const QString &)
-{
-}
