@@ -249,8 +249,8 @@ void MainApplication::loadSettings()
 
 void MainApplication::quitApplication()
 {
-  delete networkManager_;
   delete mainWindow_;
+  delete networkManager_;
 
   qWarning() << "Quit application";
 
@@ -464,7 +464,7 @@ UpdateFeeds *MainApplication::updateFeeds()
 
 void MainApplication::runUserFilter(int feedId, int filterId)
 {
-  updateFeeds_->parseObject_->runUserFilter(feedId, filterId);
+  emit signalRunUserFilter(feedId, filterId);
 }
 
 void MainApplication::sqlQueryExec(const QString &query)
