@@ -1,6 +1,6 @@
 /* ============================================================
 * QuiteRSS is a open-source cross-platform RSS/Atom news feeds reader
-* Copyright (C) 2011-2013 QuiteRSS Team <quiterssteam@gmail.com>
+* Copyright (C) 2011-2014 QuiteRSS Team <quiterssteam@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,25 +16,30 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
 #include "customizetoolbardialog.h"
-#include "rsslisting.h"
+
+#include "mainapplication.h"
+#include "settings.h"
 
 CustomizeToolbarDialog::CustomizeToolbarDialog(QWidget *parent, QToolBar *toolbar)
   : Dialog(parent),
     toolbar_(toolbar)
 {
   setWindowFlags (windowFlags() & ~Qt::WindowContextHelpButtonHint);
-  setMinimumWidth(300);
+  setMinimumWidth(400);
   setMinimumHeight(400);
 
-  if (toolbar_->objectName() == "ToolBar_General")
+  Settings settings;
+  QString iconStr;
+  if (toolbar_->objectName() == "ToolBar_General") {
     setWindowTitle(tr("Customize Main Toolbar"));
-  else if (toolbar_->objectName() == "feedsToolBar")
+    iconStr = settings.value("Settings/toolBarIconSize", "toolBarIconNormal_").toString();
+  } else if (toolbar_->objectName() == "feedsToolBar") {
     setWindowTitle(tr("Customize Feeds Toolbar"));
-  else if (toolbar_->objectName() == "newsToolBar")
+    iconStr = settings.value("Settings/feedsToolBarIconSize", "toolBarIconSmall_").toString();
+  } else if (toolbar_->objectName() == "newsToolBar") {
     setWindowTitle(tr("Customize News Toolbar"));
-
-  RSSListing *rssl_ = qobject_cast<RSSListing*>(parentWidget());
-  settings_ = rssl_->settings_;
+    iconStr = settings.value("Settings/newsToolBarIconSize", "toolBarIconSmall_").toString();
+  }
 
   shortcutTree_ = new QTreeWidget(this);
   shortcutTree_->setObjectName("actionTree");
@@ -91,7 +96,7 @@ CustomizeToolbarDialog::CustomizeToolbarDialog(QWidget *parent, QToolBar *toolba
   treeItem.clear();
   treeItem << tr("Icon") << tr("Text") << tr("Text Beside Icon") << tr("Text Under Icon");
   styleBox_->addItems(treeItem);
-  QString styleStr = settings_->value("Settings/toolBarStyle", "toolBarStyleTuI_").toString();
+  QString styleStr = settings.value("Settings/toolBarStyle", "toolBarStyleTuI_").toString();
   if (styleStr == "toolBarStyleI_") {
     styleBox_->setCurrentIndex(0);
   } else if (styleStr == "toolBarStyleT_") {
@@ -106,7 +111,6 @@ CustomizeToolbarDialog::CustomizeToolbarDialog(QWidget *parent, QToolBar *toolba
   treeItem.clear();
   treeItem << tr("Big") << tr("Normal") << tr("Small");
   iconBox_->addItems(treeItem);
-  QString iconStr = settings_->value("Settings/toolBarIconSize", "toolBarIconNormal_").toString();
   if (iconStr == "toolBarIconBig_") {
     iconBox_->setCurrentIndex(0);
   } else if (iconStr == "toolBarIconSmall_") {
@@ -115,25 +119,32 @@ CustomizeToolbarDialog::CustomizeToolbarDialog(QWidget *parent, QToolBar *toolba
     iconBox_->setCurrentIndex(1);
   }
 
+  QHBoxLayout *iconLayout = new QHBoxLayout();
+  iconLayout->addWidget(new QLabel(tr("Icon Size:")));
+  iconLayout->addWidget(iconBox_);
+  QWidget *iconWidget = new QWidget(this);
+  iconWidget->setLayout(iconLayout);
+
   QHBoxLayout *styleLayout = new QHBoxLayout();
-  styleLayout->setMargin(0);
   styleLayout->addWidget(new QLabel(tr("Style:")));
   styleLayout->addWidget(styleBox_);
-  styleLayout->addSpacing(10);
-  styleLayout->addWidget(new QLabel(tr("Icon Size:")));
-  styleLayout->addWidget(iconBox_);
-  styleLayout->addStretch();
-
   QWidget *styleWidget = new QWidget(this);
   styleWidget->setLayout(styleLayout);
 
+  QHBoxLayout *settingsLayout = new QHBoxLayout();
+  settingsLayout->setMargin(0);
+  settingsLayout->addWidget(iconWidget);
+  settingsLayout->addSpacing(10);
+  settingsLayout->addWidget(styleWidget);
+  settingsLayout->addStretch();
+
   if (toolbar_->objectName() != "ToolBar_General") {
     styleWidget->hide();
   }
 
   QVBoxLayout *mainVLayout = new QVBoxLayout();
   mainVLayout->addWidget(shortcutTree_, 1);
-  mainVLayout->addWidget(styleWidget);
+  mainVLayout->addLayout(settingsLayout);
 
   addButtonMenu_ = new QMenu(this);
   addButton_ = new QPushButton(tr("Add"));
@@ -182,16 +193,17 @@ CustomizeToolbarDialog::CustomizeToolbarDialog(QWidget *parent, QToolBar *toolba
           this, SLOT(slotCurrentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)));
   connect(this, SIGNAL(finished(int)), this, SLOT(closeDialog()));
 
-  restoreGeometry(settings_->value("customizeToolbarDlg/geometry").toByteArray());
+  restoreGeometry(settings.value("customizeToolbarDlg/geometry").toByteArray());
 }
 
 void CustomizeToolbarDialog::acceptDialog()
 {
-  RSSListing *rssl_ = qobject_cast<RSSListing*>(parentWidget());
+  Settings settings;
+  MainWindow *mainWindow = mainApp->mainWindow();
 
   if (toolbar_->objectName() == "newsToolBar") {
-    for (int i = 0; i < rssl_->stackedWidget_->count(); i++) {
-      NewsTabWidget *widget = (NewsTabWidget*)rssl_->stackedWidget_->widget(i);
+    for (int i = 0; i < mainWindow->stackedWidget_->count(); i++) {
+      NewsTabWidget *widget = (NewsTabWidget*)mainWindow->stackedWidget_->widget(i);
       QListIterator<QAction *> iter(widget->newsToolBar_->actions());
       while (iter.hasNext()) {
         QAction *pAction = iter.next();
@@ -221,7 +233,7 @@ void CustomizeToolbarDialog::acceptDialog()
       continue;
     }
 
-    QListIterator<QAction *> iter(rssl_->actions());
+    QListIterator<QAction *> iter(mainWindow->actions());
     while (iter.hasNext()) {
       QAction *pAction = iter.next();
       if (!pAction->icon().isNull()) {
@@ -238,7 +250,7 @@ void CustomizeToolbarDialog::acceptDialog()
       if (actionStr == "Separator") {
         toolbar_->addSeparator();
       } else {
-        QListIterator<QAction *> iter(rssl_->actions());
+        QListIterator<QAction *> iter(mainWindow->actions());
         while (iter.hasNext()) {
           QAction *pAction = iter.next();
           if (!pAction->icon().isNull()) {
@@ -253,7 +265,7 @@ void CustomizeToolbarDialog::acceptDialog()
   }
 
   if (toolbar_->objectName() == "ToolBar_General") {
-    settings_->setValue("Settings/mainToolBar", str);
+    settings.setValue("Settings/mainToolBar", str);
 
     switch (styleBox_->currentIndex()) {
     case 0: str = "toolBarStyleI_"; break;
@@ -261,28 +273,36 @@ void CustomizeToolbarDialog::acceptDialog()
     case 2: str = "toolBarStyleTbI_"; break;
     default: str = "toolBarStyleTuI_";
     }
-    settings_->setValue("Settings/toolBarStyle", str);
-    rssl_->setToolBarStyle(str);
+    settings.setValue("Settings/toolBarStyle", str);
+    mainWindow->setToolBarStyle(str);
 
     switch (iconBox_->currentIndex()) {
     case 0: str = "toolBarIconBig_"; break;
     case 2: str = "toolBarIconSmall_"; break;
     default: str = "toolBarIconNormal_";
     }
-    settings_->setValue("Settings/toolBarIconSize", str);
-    rssl_->setToolBarIconSize(str);
+    settings.setValue("Settings/toolBarIconSize", str);
+    mainWindow->setToolBarIconSize(toolbar_, str);
   } else if (toolbar_->objectName() == "feedsToolBar") {
-    settings_->setValue("Settings/feedsToolBar", str);
-  } else if (toolbar_->objectName() == "newsToolBar") {
-    settings_->setValue("Settings/newsToolBar", str);
+    settings.setValue("Settings/feedsToolBar2", str);
 
-    for (int i = 0; i < rssl_->stackedWidget_->count(); i++) {
-      NewsTabWidget *widget = (NewsTabWidget*)rssl_->stackedWidget_->widget(i);
+    switch (iconBox_->currentIndex()) {
+    case 0: str = "toolBarIconBig_"; break;
+    case 2: str = "toolBarIconSmall_"; break;
+    default: str = "toolBarIconNormal_";
+    }
+    settings.setValue("Settings/feedsToolBarIconSize", str);
+    mainWindow->setToolBarIconSize(toolbar_, str);
+  } else if (toolbar_->objectName() == "newsToolBar") {
+    settings.setValue("Settings/newsToolBar", str);
+
+    for (int i = 0; i < mainWindow->stackedWidget_->count(); i++) {
+      NewsTabWidget *widget = (NewsTabWidget*)mainWindow->stackedWidget_->widget(i);
       foreach (QString actionStr, str.split(",", QString::SkipEmptyParts)) {
         if (actionStr == "Separator") {
           widget->newsToolBar_->addSeparator();
         } else {
-          QListIterator<QAction *> iter(rssl_->actions());
+          QListIterator<QAction *> iter(mainWindow->actions());
           while (iter.hasNext()) {
             QAction *pAction = iter.next();
             if (!pAction->icon().isNull()) {
@@ -295,7 +315,15 @@ void CustomizeToolbarDialog::acceptDialog()
         }
       }
       widget->newsToolBar_->addAction(widget->separatorRAct_);
-      widget->newsToolBar_->addAction(rssl_->restoreNewsAct_);
+      widget->newsToolBar_->addAction(mainWindow->restoreNewsAct_);
+
+      switch (iconBox_->currentIndex()) {
+      case 0: str = "toolBarIconBig_"; break;
+      case 2: str = "toolBarIconSmall_"; break;
+      default: str = "toolBarIconNormal_";
+      }
+      settings.setValue("Settings/newsToolBarIconSize", str);
+      mainWindow->setToolBarIconSize(widget->newsToolBar_, str);
     }
   }
 
@@ -304,7 +332,8 @@ void CustomizeToolbarDialog::acceptDialog()
 
 void CustomizeToolbarDialog::closeDialog()
 {
-  settings_->setValue("customizeToolbarDlg/geometry", saveGeometry());
+  Settings settings;
+  settings.setValue("customizeToolbarDlg/geometry", saveGeometry());
 }
 
 void CustomizeToolbarDialog::slotCurrentItemChanged(QTreeWidgetItem *current,
@@ -331,8 +360,7 @@ void CustomizeToolbarDialog::showMenuAddButton()
 {
   addButtonMenu_->clear();
 
-  RSSListing *rssl_ = qobject_cast<RSSListing*>(parentWidget());
-  QListIterator<QAction *> iter(rssl_->actions());
+  QListIterator<QAction *> iter(mainApp->mainWindow()->actions());
   while (iter.hasNext()) {
     QAction *pAction = iter.next();
     if ((pAction->objectName() == "restoreNewsAct") ||
@@ -437,8 +465,7 @@ void CustomizeToolbarDialog::defaultShortcut()
       item->setBackground(0, qApp->palette().brush(QPalette::Midlight));
       shortcutTree_->addTopLevelItem(item);
     } else {
-      RSSListing *rssl_ = qobject_cast<RSSListing*>(parentWidget());
-      QListIterator<QAction *> iter(rssl_->actions());
+      QListIterator<QAction *> iter(mainApp->mainWindow()->actions());
       while (iter.hasNext()) {
         QAction *pAction = iter.next();
         if (!pAction->icon().isNull()) {
