@@ -1,6 +1,6 @@
 /* ============================================================
 * QuiteRSS is a open-source cross-platform RSS/Atom news feeds reader
-* Copyright (C) 2011-2013 QuiteRSS Team <quiterssteam@gmail.com>
+* Copyright (C) 2011-2014 QuiteRSS Team <quiterssteam@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,8 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
 #include "feedstreeview.h"
+
+#include "mainapplication.h"
 #include "feedstreemodel.h"
 #include "delegatewithoutfocus.h"
 
@@ -27,17 +29,18 @@ FeedsTreeView::FeedsTreeView(QWidget * parent)
   : QyurSqlTreeView(parent)
   , selectIdEn_(true)
   , autocollapseFolder_(false)
+  , sourceModel_(0)
   , dragPos_(QPoint())
   , dragStartPos_(QPoint())
-  , selectOldId_(-1)
+  , expandedOldId_(-1)
 {
   setObjectName("feedsTreeView_");
+  setFrameStyle(QFrame::NoFrame);
   setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
   setEditTriggers(QAbstractItemView::NoEditTriggers);
 
   setSelectionBehavior(QAbstractItemView::SelectRows);
-//  setSelectionMode(QAbstractItemView::ExtendedSelection);
-  setSelectionMode(QAbstractItemView::SingleSelection);
+  setSelectionMode(QAbstractItemView::ExtendedSelection);
   setUniformRowHeights(true);
 
   header()->setStretchLastSection(false);
@@ -53,16 +56,113 @@ FeedsTreeView::FeedsTreeView(QWidget * parent)
   setAcceptDrops(true);
   setDropIndicatorShown(true);
 
-  connect(this, SIGNAL(expanded(const QModelIndex&)), SLOT(slotExpanded(const QModelIndex&)));
-  connect(this, SIGNAL(collapsed(const QModelIndex&)), SLOT(slotCollapsed(const QModelIndex&)));
+  connect(this, SIGNAL(expanded(QModelIndex)), SLOT(slotExpanded(QModelIndex)));
+  connect(this, SIGNAL(collapsed(QModelIndex)), SLOT(slotCollapsed(QModelIndex)));
+}
+
+void FeedsTreeView::setSourceModel(FeedsTreeModel *sourceModel)
+{
+  sourceModel_ = sourceModel;
+  QyurSqlTreeView::setSourceModel(sourceModel_);
+
+  QSqlQuery q;
+  q.exec("SELECT id FROM feeds WHERE f_Expanded=1 AND (xmlUrl='' OR xmlUrl IS NULL)");
+  while (q.next()) {
+    int feedId = q.value(0).toInt();
+    expandedList.append(feedId);
+  }
+}
+
+void FeedsTreeView::refresh()
+{
+  sourceModel_->refresh();
+  ((FeedsProxyModel*)model())->reset();
+  restoreExpanded();
+}
+
+bool FeedsTreeView::isFolder(const QModelIndex &index) const
+{
+  if (!index.isValid())
+    return false;
+  return sourceModel_->isFolder(((FeedsProxyModel*)model())->mapToSource(index));
+}
+
+void FeedsTreeView::restoreExpanded()
+{
+  foreach (int id, expandedList) {
+    QModelIndex index = ((FeedsProxyModel*)model())->mapFromSource(id);
+    if (!isExpanded(index))
+      setExpanded(index, true);
+  }
+}
+
+/** @brief Process item expanding
+ *----------------------------------------------------------------------------*/
+void FeedsTreeView::slotExpanded(const QModelIndex &index)
+{
+  int feedId = sourceModel_->getIdByIndex(((FeedsProxyModel*)model())->mapToSource(index));
+  if (!expandedList.contains(feedId)) {
+    expandedList.append(feedId);
+    mainApp->sqlQueryExec(QString("UPDATE feeds SET f_Expanded=1 WHERE id=='%1'").arg(feedId));
+  }
+
+  if (!autocollapseFolder_ || (feedId == expandedOldId_))
+    return;
+
+  QModelIndex indexCollapsed = ((FeedsProxyModel*)model())->mapFromSource(expandedOldId_);
+  expandedOldId_ = feedId;
+
+  if (index.parent() != indexCollapsed)
+    collapse(indexCollapsed);
+
+  int value = index.row();
+  QModelIndex parent = index.parent();
+  if (parent.isValid()) value = value + 1;
+  while (parent.isValid()) {
+    value = value + parent.row();
+    parent = parent.parent();
+  }
+  verticalScrollBar()->setValue(value);
+}
+
+/** @brief Process item collapsing
+ *----------------------------------------------------------------------------*/
+void FeedsTreeView::slotCollapsed(const QModelIndex &index)
+{
+  int feedId = sourceModel_->getIdByIndex(((FeedsProxyModel*)model())->mapToSource(index));
+  expandedList.removeOne(feedId);
+  mainApp->sqlQueryExec(QString("UPDATE feeds SET f_Expanded=0 WHERE id=='%1'").arg(feedId));
+}
+
+void FeedsTreeView::expandAll()
+{
+  expandedList.clear();
+  QyurSqlTreeView::expandAll();
+
+
+  QSqlQuery q;
+  q.exec("SELECT id FROM feeds WHERE (xmlUrl='' OR xmlUrl IS NULL)");
+  while (q.next()) {
+    int feedId = q.value(0).toInt();
+    expandedList.append(feedId);
+  }
+  mainApp->sqlQueryExec("UPDATE feeds SET f_Expanded=1 WHERE (xmlUrl='' OR xmlUrl IS NULL)");
+}
+
+void FeedsTreeView::collapseAll()
+{
+  expandedList.clear();
+  QyurSqlTreeView::collapseAll();
+
+  mainApp->sqlQueryExec("UPDATE feeds SET f_Expanded=0 WHERE (xmlUrl='' OR xmlUrl IS NULL)");
 }
 
 /** @brief Find index of next unread (by meaning) feed
  * @details Find unread feed with condition.
  * @param indexCur Index to start search from
  * @param nextCondition Find condition:
- *  1 - find previous,
- *  2 - find next,
+ *  2 - find previous,
+ *  1 - find next,
  *  0 - find next. If fails, find previous (default)
  * @return finded index or QModelIndex()
  *---------------------------------------------------------------------------*/
@@ -72,7 +172,7 @@ QModelIndex FeedsTreeView::indexNextUnread(const QModelIndex &indexCur, int next
     // find next
     QModelIndex index = indexNext(indexCur);
     while (index.isValid()) {
-      int feedUnreadCount = ((FeedsTreeModel*)model())->dataField(index, "unread").toInt();
+      int feedUnreadCount = sourceModel_->dataField(((FeedsProxyModel*)model())->mapToSource(index), "unread").toInt();
       if (0 < feedUnreadCount)
         return index;  // ok
 
@@ -84,7 +184,7 @@ QModelIndex FeedsTreeView::indexNextUnread(const QModelIndex &indexCur, int next
     // find previous
     QModelIndex index = indexPrevious(indexCur);
     while (index.isValid()) {
-      int feedUnreadCount = ((FeedsTreeModel*)model())->dataField(index, "unread").toInt();
+      int feedUnreadCount = sourceModel_->dataField(((FeedsProxyModel*)model())->mapToSource(index), "unread").toInt();
       if (0 < feedUnreadCount)
         return index;  // ok
 
@@ -102,8 +202,8 @@ QModelIndex FeedsTreeView::lastFeedInFolder(const QModelIndex &indexFolder)
   QModelIndex index = QModelIndex();
 
   for(int i = model()->rowCount(indexFolder)-1; i >= 0; --i) {
-    index = indexFolder.child(i, indexFolder.column());
-    if (((FeedsTreeModel*)model())->isFolder(index))
+    index = indexFolder.child(i, columnIndex("text"));
+    if (isFolder(index))
       index = lastFeedInFolder(index);
     if (index.isValid())
       break;
@@ -120,15 +220,15 @@ QModelIndex FeedsTreeView::lastFeedInFolder(const QModelIndex &indexFolder)
 QModelIndex FeedsTreeView::indexPrevious(const QModelIndex &indexCur, bool isParent)
 {
   QModelIndex index = QModelIndex();
-  if (((FeedsTreeModel*)model())->isFolder(indexCur) && !isParent) {
+  if (isFolder(indexCur) && !isParent) {
     index = lastFeedInFolder(indexCur);
     if (index.isValid())
       return index;
   }
 
   for(int i = indexCur.row()-1; i >= 0; --i) {
-    index = model()->index(i, indexCur.column(), indexCur.parent());
-    if (((FeedsTreeModel*)model())->isFolder(index))
+    index = model()->index(i, columnIndex("text"), indexCur.parent());
+    if (isFolder(index))
       index = lastFeedInFolder(index);
     if (index.isValid())
       return index;
@@ -147,8 +247,8 @@ QModelIndex FeedsTreeView::firstFeedInFolder(const QModelIndex &indexFolder)
   QModelIndex index = QModelIndex();
 
   for(int i = 0; i < model()->rowCount(indexFolder); i++) {
-    index = indexFolder.child(i, indexFolder.column());
-    if (((FeedsTreeModel*)model())->isFolder(index))
+    index = indexFolder.child(i, columnIndex("text"));
+    if (isFolder(index))
       index = firstFeedInFolder(index);
     if (index.isValid())
       break;
@@ -165,7 +265,7 @@ QModelIndex FeedsTreeView::firstFeedInFolder(const QModelIndex &indexFolder)
 QModelIndex FeedsTreeView::indexNext(const QModelIndex &indexCur, bool isParent)
 {
   QModelIndex index = QModelIndex();
-  if (((FeedsTreeModel*)model())->isFolder(indexCur) && !isParent) {
+  if (isFolder(indexCur) && !isParent) {
     index = firstFeedInFolder(indexCur);
     if (index.isValid())
       return index;
@@ -174,8 +274,8 @@ QModelIndex FeedsTreeView::indexNext(const QModelIndex &indexCur, bool isParent)
   int rowCount = model()->rowCount(indexCur.parent());
 
   for(int i = indexCur.row()+1; i < rowCount; i++) {
-    index = model()->index(i, indexCur.column(), indexCur.parent());
-    if (((FeedsTreeModel*)model())->isFolder(index))
+    index = model()->index(i, columnIndex("text"), indexCur.parent());
+    if (isFolder(index))
       index = firstFeedInFolder(index);
     if (index.isValid())
       return index;
@@ -193,15 +293,15 @@ QModelIndex FeedsTreeView::lastFolderInFolder(const QModelIndex &indexFolder)
 {
   if (indexFolder.isValid()) {
     for(int i = model()->rowCount(indexFolder)-1; i >= 0; --i) {
-      QModelIndex index = indexFolder.child(i, indexFolder.column());
-      if (((FeedsTreeModel*)model())->isFolder(index)) {
+      QModelIndex index = indexFolder.child(i, columnIndex("text"));
+      if (isFolder(index)) {
         return index;
       }
     }
   } else {
     for(int i = model()->rowCount(indexFolder)-1; i >= 0; --i) {
       QModelIndex index = model()->index(i, columnIndex("text"));
-      if (((FeedsTreeModel*)model())->isFolder(index))
+      if (isFolder(index))
         return index;
     }
   }
@@ -215,8 +315,8 @@ QModelIndex FeedsTreeView::indexPreviousFolder(const QModelIndex &indexCur)
   QModelIndex index = QModelIndex();
 
   for(int i = indexCur.row()-1; i >= 0; --i) {
-    index = model()->index(i, indexCur.column(), indexCur.parent());
-    if (((FeedsTreeModel*)model())->isFolder(index))
+    index = model()->index(i, columnIndex("text"), indexCur.parent());
+    if (isFolder(index))
       return index;
   }
 
@@ -232,15 +332,15 @@ QModelIndex FeedsTreeView::firstFolderInFolder(const QModelIndex &indexFolder)
 {
   if (indexFolder.isValid()) {
     for(int i = 0; i < model()->rowCount(indexFolder); i++) {
-      QModelIndex index = indexFolder.child(i, indexFolder.column());
-      if (((FeedsTreeModel*)model())->isFolder(index)) {
+      QModelIndex index = indexFolder.child(i, columnIndex("text"));
+      if (isFolder(index)) {
         return index;
       }
     }
   } else {
     for(int i = 0; i < model()->rowCount(indexFolder); i++) {
       QModelIndex index = model()->index(i, columnIndex("text"));
-      if (((FeedsTreeModel*)model())->isFolder(index))
+      if (isFolder(index))
         return index;
     }
   }
@@ -252,7 +352,7 @@ QModelIndex FeedsTreeView::firstFolderInFolder(const QModelIndex &indexFolder)
 QModelIndex FeedsTreeView::indexNextFolder(const QModelIndex &indexCur, bool isParent)
 {
   QModelIndex index = QModelIndex();
-  if ((((FeedsTreeModel*)model())->isFolder(indexCur) && !isParent)) {
+  if ((isFolder(indexCur) && !isParent)) {
     index = firstFolderInFolder(indexCur);
     if (index.isValid()) {
       return index;
@@ -261,8 +361,8 @@ QModelIndex FeedsTreeView::indexNextFolder(const QModelIndex &indexCur, bool isP
 
   int rowCount = model()->rowCount(indexCur.parent());
   for(int i = indexCur.row()+1; i < rowCount; i++) {
-    index = model()->index(i, indexCur.column(), indexCur.parent());
-    if (((FeedsTreeModel*)model())->isFolder(index))
+    index = model()->index(i, columnIndex("text"), indexCur.parent());
+    if (isFolder(index))
       return index;
   }
 
@@ -289,7 +389,8 @@ void FeedsTreeView::mousePressEvent(QMouseEvent *event)
 
   if (event->buttons() & Qt::RightButton) {
     if (event->pos().x() >= rectText.x()) {
-      selectId_ = ((FeedsTreeModel*)model())->getIdByIndex(index);
+      index = ((FeedsProxyModel*)model())->mapToSource(index);
+      selectId_ = sourceModel_->getIdByIndex(index);
     }
     return;
   }
@@ -301,7 +402,8 @@ void FeedsTreeView::mousePressEvent(QMouseEvent *event)
   }
 
   indexClicked_ = index;
-  selectId_ = ((FeedsTreeModel*)model())->getIdByIndex(index);
+  index = ((FeedsProxyModel*)model())->mapToSource(index);
+  selectId_ = sourceModel_->getIdByIndex(index);
 
   if ((event->buttons() & Qt::MiddleButton)) {
     emit signalMiddleClicked();
@@ -348,7 +450,7 @@ void FeedsTreeView::mouseReleaseEvent(QMouseEvent *event)
 
   if (!index.isValid()) return;
   if (!(event->pos().x() >= rectText.x()) ||
-      (((FeedsTreeModel*)model())->isFolder(index))) {
+      (isFolder(index))) {
     QyurSqlTreeView::mouseDoubleClickEvent(event);
     return;
   }
@@ -375,8 +477,8 @@ void FeedsTreeView::mouseReleaseEvent(QMouseEvent *event)
                                            const QModelIndex &previous)
 {
   if (selectIdEn_) {
-    QModelIndex index = current;
-    selectId_ = ((FeedsTreeModel*)model())->getIdByIndex(index);
+    QModelIndex index = ((FeedsProxyModel*)model())->mapToSource(current);
+    selectId_ = sourceModel_->getIdByIndex(index);
   }
   selectIdEn_ = true;
 
@@ -412,16 +514,18 @@ void FeedsTreeView::dragMoveEvent(QDragMoveEvent *event)
   QModelIndex dragIndex = indexAt(dragPos_);
 
   // Process categories
-  if (((FeedsTreeModel*)model())->isFolder(dragIndex)) {
+  if (isFolder(dragIndex)) {
     if (dragIndex == currentIndex().parent())
       event->ignore();  // drag-to-category is parent already dragged one
-    else if (dragIndex == currentIndex())
+    else if ((dragIndex.row() == currentIndex().row()) &&
+             (dragIndex.parent() == currentIndex().parent()))
       event->ignore();  // don't move category to itself
     else {
       bool ignore = false;
       QModelIndex child = dragIndex.parent();
       while (child.isValid()) {
-        if (child == currentIndex()) {
+        if ((child.row() == currentIndex().row()) &&
+            (child.parent() == currentIndex().parent())) {
           event->ignore();  // don't move category inside itself
           ignore = true;
           break;
@@ -433,7 +537,8 @@ void FeedsTreeView::dragMoveEvent(QDragMoveEvent *event)
   }
   // Process feeds
   else {
-    if (dragIndex == currentIndex())
+    if ((dragIndex.row() == currentIndex().row()) &&
+        (dragIndex.parent() == currentIndex().parent()))
       event->ignore();  // don't move feed to itseelf
     else if (dragIndex.parent() == currentIndex())
       event->ignore();  // don't move feed to same parent
@@ -441,7 +546,8 @@ void FeedsTreeView::dragMoveEvent(QDragMoveEvent *event)
       bool ignore = false;
       QModelIndex child = dragIndex.parent();
       while (child.isValid()) {
-        if (child == currentIndex()) {
+        if ((child.row() == currentIndex().row()) &&
+            (child.parent() == currentIndex().parent())) {
           event->ignore();  // don't move feed inside itself
           ignore = true;
           break;
@@ -470,50 +576,6 @@ bool FeedsTreeView::shouldAutoScroll(const QPoint &pos) const
         || (area.right() - pos.x() < autoScrollMargin());
 }
 
-/** @brief Process item expanding
- *----------------------------------------------------------------------------*/
-void FeedsTreeView::slotExpanded(const QModelIndex &index)
-{
-  QModelIndex indexExpanded = model()->index(index.row(), columnIndex("f_Expanded"), index.parent());
-  model()->setData(indexExpanded, 1);
-
-  int feedId = ((FeedsTreeModel*)model())->getIdByIndex(indexExpanded);
-  QSqlQuery q;
-  q.exec(QString("UPDATE feeds SET f_Expanded=1 WHERE id=='%2'").arg(feedId));
-
-  if (feedId == selectOldId_) return;
-
-  QModelIndex indexCollapsed =
-      ((FeedsTreeModel*)model())->getIndexById(selectOldId_);
-  selectOldId_ = feedId;
-
-  if (!autocollapseFolder_) return;
-
-  if (indexExpanded.parent() != indexCollapsed)
-    collapse(indexCollapsed);
-
-  int value = index.row();
-  QModelIndex parent = index.parent();
-  if (parent.isValid()) value = value + 1;
-  while (parent.isValid()) {
-    value = value + parent.row();
-    parent = parent.parent();
-  }
-  verticalScrollBar()->setValue(value);
-}
-
-/** @brief Process item collapsing
- *----------------------------------------------------------------------------*/
-void FeedsTreeView::slotCollapsed(const QModelIndex &index)
-{
-  QModelIndex indexExpanded = model()->index(index.row(), columnIndex("f_Expanded"), index.parent());
-  model()->setData(indexExpanded, 0);
-
-  int feedId = ((FeedsTreeModel*)model())->getIdByIndex(indexExpanded);
-  QSqlQuery q;
-  q.exec(QString("UPDATE feeds SET f_Expanded=0 WHERE id=='%2'").arg(feedId));
-}
-
 // ----------------------------------------------------------------------------
 void FeedsTreeView::dropEvent(QDropEvent *event)
 {
@@ -535,35 +597,41 @@ void FeedsTreeView::paintEvent(QPaintEvent *event)
   QModelIndex dragIndex = indexAt(dragPos_);
 
   // Process folders
-  if (((FeedsTreeModel*)model())->isFolder(dragIndex)) {
+  if (isFolder(dragIndex)) {
     if ((dragIndex == currentIndex().parent()) ||
-        (dragIndex == currentIndex()))
+        ((dragIndex.row() == currentIndex().row()) &&
+         (dragIndex.parent() == currentIndex().parent())))
       return;
 
     QModelIndex child = dragIndex.parent();
     while (child.isValid()) {
-      if (child == currentIndex()) return;
+      if ((child.row() == currentIndex().row()) &&
+          (child.parent() == currentIndex().parent()))
+        return;
       child = child.parent();
     }
   }
   // Process feeds
   else {
-    if ((dragIndex == currentIndex()) ||
-        (dragIndex.parent() == currentIndex()))
+    if ((dragIndex.parent() == currentIndex()) ||
+        ((dragIndex.row() == currentIndex().row()) &&
+         (dragIndex.parent() == currentIndex().parent())))
       return;
     QModelIndex child = dragIndex.parent();
     while (child.isValid()) {
-      if (child == currentIndex()) return;
+      if ((child.row() == currentIndex().row()) &&
+          (child.parent() == currentIndex().parent()))
+        return;
       child = child.parent();
     }
   }
 
   QModelIndex indexText = model()->index(dragIndex.row(),
-                                         ((QyurSqlTreeModel*)model())->proxyColumnByOriginal("text"),
+                                         columnIndex("text"),
                                          dragIndex.parent());
 
   QRect rectText = visualRect(indexText);
-  rectText.setWidth(rectText.width()-1);
+  rectText.setRight(viewport()->width()-2);
   QBrush brush = qApp->palette().brush(QPalette::Highlight);
 
   QPainter painter;
@@ -589,7 +657,7 @@ void FeedsTreeView::paintEvent(QPaintEvent *event)
                      viewport()->width()-2, rectText.bottom()+2);
   }
   else {
-    if (!((FeedsTreeModel*)model())->isFolder(dragIndex))
+    if (!isFolder(dragIndex))
       return;
 
     painter.begin(this->viewport());
@@ -609,7 +677,7 @@ void FeedsTreeView::paintEvent(QPaintEvent *event)
 // ----------------------------------------------------------------------------
 QPersistentModelIndex FeedsTreeView::selectIndex()
 {
-  return ((FeedsTreeModel*)model())->getIndexById(selectId_);
+  return sourceModel_->getIndexById(selectId_);
 }
 
 /** @brief Update cursor without list scrolling
@@ -638,15 +706,16 @@ void FeedsTreeView::handleDrop(QDropEvent *e)
   } else if (qAbs(rectText.bottom() - e->pos().y()) < 3) {
     how = 1;
   } else {
-    if (((FeedsTreeModel*)model())->isFolder(dropIndex)) {
+    if (isFolder(dropIndex)) {
       how = 2;
     } else {
       dropIndex = model()->index(dropIndex.row()+1,
-                                  ((QyurSqlTreeModel*)model())->proxyColumnByOriginal("text"),
+                                  columnIndex("text"),
                                   dropIndex.parent());
       if (!dropIndex.isValid()) how = 1;
     }
   }
-
+  indexWhat = ((FeedsProxyModel*)model())->mapToSource(indexWhat);
+  indexWhere = ((FeedsProxyModel*)model())->mapToSource(indexWhere);
   emit signalDropped(indexWhat, indexWhere, how);
 }
